#!/usr/bin/python
######################################################################
## Filename:      make-spheres-inc.py
## Description:   MOVIE 1 / Rotate and stereographically project
##                the centers, generate POV-Ray sphere primitives
## Author:        David Dumas <david@dumas.io>
## Modified at:   Wed Nov 28 14:40:33 2012
##                
## Copyright (C) 2012 David Dumas
##                
## This program is free software distributed under the GNU General
## Public License (http://www.gnu.org/licenses/gpl.txt).
######################################################################

# Expected input is in the form of "lenderiv.py" output, lines like
# aDCBdbcd 14.66 -0.57 0.00 -0.87 -1.02
# <word> <curlen> <dlen/dx1> <dlen/dx2> ...

import sys
from numpy import *
from numpy import linalg

t = float(sys.argv[1])

def rot(theta):
    return matrix( [[ cos(theta), -sin(theta), 0, 0],
                    [ sin(theta), cos(theta), 0, 0],
                    [ 0, 0, 1, 0],
                    [ 0, 0, 0, 1]] )


def normalize(v):
    w = (1.0 / linalg.norm(v))*v
    return w

def stereo(v):
    return array([ v[0], v[2], v[3] ]) / (1.0 - v[1])

def transform(T,v):
    return array(transpose((T * transpose(matrix(v)))))[0]

def project(v,theta):
    return stereo(transform(rot(theta),normalize(v)))

maxr = 0.0
for line in sys.stdin:
    line = line.strip()
    if not line:
        continue
    if line[0] == '#':
        continue
    fields = line.split()
    w = fields[0]
    L = float(fields[1])
    p = array([float(x) for x in fields[2:]])
#    r = 0.7489015770594103538 * 0.6 / pow(L,1.0)
    r = 0.6 / pow(L,1.2)
    if r > maxr:
        maxr = r
    C = project(p,t)
    print 'sphere { <%f,%f,%f>, %f }' % (C[0],C[1],C[2],r)

sys.stderr.write('maxr = %f\n' % maxr)
